#!/usr/bin/env python
"""
Wrapper for docker binary

- If we are docker-run'ing a trusted image, add capabilities

To enable, copy the program and set the following Condor config:
DOCKER = /usr/local/libexec/condor-docker
"""

import argparse
import subprocess
import sys
import syslog
import re

DOCKER_BIN = '/usr/bin/docker'
TRUSTED_IMAGES = [
    'unlhcc/osg-wn-el6',
    'docker.io/unlhcc/osg-wn-el6',
    'unlhcc/osg-wn-el7',
    'docker.io/unlhcc/osg-wn-el7',
    'mschnepf/slc6-condocker',
    'mschnepf/slc7-condocker',
    'pkonst/gridka-wn-sl6',
    'gitlab-registry.cern.ch/batch-team/slc6-batchwn',
]
TRUSTED_CAPABILITIES = [
    '--cap-add=DAC_OVERRIDE',
    '--cap-add=SETUID',
    '--cap-add=SETGID',
    '--cap-add=SYS_ADMIN',
    '--cap-add=SYS_CHROOT',
    '--cap-add=SYS_PTRACE',
]

class ArgumentParser(argparse.ArgumentParser):
    """Don't exit on argparse error"""
    # https://www.python.org/dev/peps/pep-0389/#discussion-sys-stderr-and-sys-exit
    def error(self, message):
        pass

def main():
    """Main"""
    parser = ArgumentParser(add_help=False)
    parser.add_argument('exe')
    args, _ = parser.parse_known_args()

    # By default, we just pass on the command
    cmd = [DOCKER_BIN] + sys.argv[1:]

    # Are we handling 'docker run'?
    if args.exe in ['run', 'create']:
        # Yes, parse the options to 'docker run'
        parser = ArgumentParser(add_help=False)
        parser.add_argument('exe')
        parser.add_argument('image')
        parser.add_argument('command', nargs=argparse.REMAINDER)

        # We have to tell argparse about all the --parameters with an argument
        # which are passed from Condor to 'docker run'.
        # Otherwise we cannot identify the Docker image because it is a
        # positional argument.
        parser.add_argument('--cpu-shares', dest='cpu-shares', action='append')
        parser.add_argument('--cap-drop', dest='cap-drop', action='append')
        parser.add_argument('--hostname', action='append')
        parser.add_argument('--label', action='append')
        parser.add_argument('--name', action='append')
        parser.add_argument('--env', '-e', action='append')
        parser.add_argument('--volume', action='append')
        parser.add_argument('--workdir', action='append')
        parser.add_argument('--user', action='append')
        parser.add_argument('--memory', action='append')
        parser.add_argument('--group-add', dest='group-add', action='append')
        parser.add_argument('--gpus', action='append')
        parser.add_argument('--security-opt', dest='security-opt', action='append')

        args, docker_opts = parser.parse_known_args()

        # Put our parsed options back into the list
        for arg, vals in vars(args).items():
            if arg in ['exe', 'image', 'command']:
                continue
            for val in vals or []:
                if arg=='memory':
                    # split memory argument in numbers (amount of memory) and characters (unit), first element is an empty string
                    _, request_memory, unit = re.split('(\d+)', val)
                    docker_opts += ['--memory', "{}{}".format(int(request_memory), unit)]
                    docker_opts += ['--memory-swap', "{}{}".format(int(request_memory), unit)]
                    docker_opts += ['--memory-reservation',"{}{}".format(int(request_memory), unit)]
                    docker_opts += ['--memory-swappiness', "0"]
                else: 
                    docker_opts += ['--' + arg, val]

        # Enable GPUS if CUDA_VISIBLE_DEVICES is set to a GPU but --gpus flag is not present
        if [x for x in docker_opts if (('CUDA_VISIBLE_DEVICES=' in x) and not ('CUDA_VISIBLE_DEVICES=10000' in x))] and ('--gpus' not in docker_opts):
            docker_opts += ['--gpus', 'all']

        # Did we successfully parse the docker run command?
        if args.image and args.command:
            # Is the Docker image in the trusted list?
            if args.image in TRUSTED_IMAGES:
                docker_opts += TRUSTED_CAPABILITIES

            # Build the new 'docker create' command
            cmd = [DOCKER_BIN, args.exe] + docker_opts + [args.image] + args.command
    else:
        pass

    syslog.syslog('"{}" => "{}"'.format(' '.join(sys.argv), ' '.join(cmd)))
    ret = subprocess.call(cmd)
    sys.exit(ret)

if __name__ == "__main__":
    main()
